﻿using System;
using System.IO;
using System.Runtime.Remoting;
using System.Web;
using System.Xml;

namespace ValidatePreviousLicenseDemo
{
    public class Validator
    {
        private const string NamespaceTypes = "http://xml.cleverbridge.com/2.170/cleverbridgeTypes.xsd";
        private const string NamespaceUpgradeManagement = "http://xml.cleverbridge.com/2.170/cleverbridgeUpgradeManagement.xsd";

        public static string GetResponseToValidationInquiry(HttpRequest validationrequest)
        {
            try
            {
                    var previousLicenseToValidate = GetLicense(validationrequest.InputStream);

                    var result = GetValidationResult(previousLicenseToValidate);

                    return GetResponse(result);
            }
            catch (Exception)
            {
                //some exception handling, 
                throw new ServerException();
            }
        }


        private static string GetLicense(Stream inputStream)
        {
            var xmlDocument = new XmlDocument();
            xmlDocument.Load(inputStream);
            var namespaceManager = new XmlNamespaceManager(xmlDocument.NameTable);
            namespaceManager.AddNamespace("cbn", NamespaceUpgradeManagement);
            namespaceManager.AddNamespace("cbt", NamespaceTypes);

            if (xmlDocument.DocumentElement != null)
            {
                var xmlElement = xmlDocument.DocumentElement.SelectSingleNode("cbn:Item/cbt:PreviousLicense", namespaceManager);
                if (xmlElement != null)
                    return xmlElement.InnerText;
            }
            return String.Empty;
        }

        private static Result GetValidationResult(string previousLicenseToValidate)
        {
            if (String.IsNullOrEmpty(previousLicenseToValidate))
            {
                //this could be your default result by error
                return new Result
                           {
                               IsValid = false,
                               ErrorId = ErrorType.KNF,
                               ErrorMessage = String.Empty
                           };

            }
            //Here some real validation
            return new Result{IsValid = true};
        }

        private static string GetResponse(Result result)
        {
            var xmlDocument = new XmlDocument();
            xmlDocument.AppendChild(xmlDocument.CreateXmlDeclaration("1.0", "UTF-8", null));

            var documentXmlNode = xmlDocument.AppendChild(xmlDocument.CreateElement("cbn", "ValidatePreviousLicenseCartItemResponse", NamespaceUpgradeManagement));
            documentXmlNode.AppendChild(xmlDocument.CreateElement("cbn", "Valid", NamespaceUpgradeManagement)).InnerText = XmlConvert.ToString(result.IsValid);

            if (!result.IsValid)
            {
                documentXmlNode.AppendChild(xmlDocument.CreateElement("cbn", "ErrorId", NamespaceUpgradeManagement)).InnerText = result.ErrorId.ToString();
                if(result.ErrorId==ErrorType.CUS)
                    documentXmlNode.AppendChild(xmlDocument.CreateElement("cbn", "Text", NamespaceUpgradeManagement)).InnerText = result.ErrorMessage;
            }
            return xmlDocument.OuterXml;
        }
    }
}